#include <string.h>
#include <unistd.h>
#include <time.h>
#include <stdio.h>
#include <openssl/md5.h>

#include "utils.h"
#include "rss.h"

char * escape_html(const char *in)
{
        int len = strlen(in) + 1;
        int out_offset = 0;
        int loop;
        char *out = (char *)mymalloc(len * 8, "escape_html");

        for(loop=0; loop<len; loop++)
        {
                if (in[loop] == '<')
                        out_offset += sprintf(&out[out_offset], "&lt;");
                else if (in[loop] == '>')
                        out_offset += sprintf(&out[out_offset], "&gt;");
                else if (in[loop] == '&')
                        out_offset += sprintf(&out[out_offset], "&amp;");
                else
                        out[out_offset++] = in[loop];
        }

        return out;
}

void rss_add_string(char **into, const char *tag, const char *what)
{
	char *escaped = escape_html(what);
	int len = strlen(*into);
	int new_len = len + strlen(tag) * 2 + 5 + strlen(escaped) + 1;

	*into = (char *)myrealloc(*into, new_len, "rss_add_string");
	sprintf(&(*into)[len], "<%s>%s</%s>", tag, escaped, tag);

	myfree(escaped, "escaped string");
}

void rss_add_section(char **into, const char *tag, int open)
{
	int len = strlen(*into);
	int new_len = len + strlen(tag) + 3 + 1;

	*into = (char *)myrealloc(*into, new_len, "rss_add_section");

	if (open)
		sprintf(&(*into)[len], "<%s>", tag);
	else
		sprintf(&(*into)[len], "</%s>", tag);
}

void rss_add_guid(char **into, const char *what, int is_perma_link)
{
	int len = strlen(*into);

	*into = (char *)myrealloc(*into, len + strlen(what) + 64, "rss_add_guid");

	sprintf(&(*into)[len], "<guid isPermaLink=\"%s\">%s</guid>", is_perma_link?"true":"false", what);
}

void rss_add_timestamp(char **into, time_t ts)
{
	char buffer[128];
        struct tm *tm_struct = localtime(&ts);
	static const char *month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	static const char *wday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

        sprintf(buffer, "%s, %02d %3s %04d %02d:%02d:%02d %s",
                        wday[tm_struct -> tm_wday],
                        tm_struct -> tm_mday,
                        month[tm_struct -> tm_mon],
                        tm_struct -> tm_year + 1900,
                        tm_struct -> tm_hour,
                        tm_struct -> tm_min,
                        tm_struct -> tm_sec,
                        "+0000"/*tzname[0]*/);

	rss_add_string(into, "pubDate", buffer);
}

char * rss_header(const char *title, const char *link, const char *description, const char *man_edit, const char *webmaster, const char *generator)
{
	const char *rss_start = "<?xml version=\"1.0\"?><rss version=\"2.0\"><channel>";
	int rss_start_len = strlen(rss_start);
	char *rss_str = (char *)mymalloc(rss_start_len + 1, "rss string");

	sprintf(rss_str, "%s", rss_start);
	rss_add_string(&rss_str, "title", title);
	rss_add_string(&rss_str, "link", link);
	rss_add_string(&rss_str, "description", description);
	rss_add_string(&rss_str, "managingEditor", man_edit);
	rss_add_string(&rss_str, "webMaster", webmaster);
	rss_add_string(&rss_str, "generator", generator);

	return rss_str;
}

void rss_item(char **into, const char *title, const char *link, const char *description, time_t timestamp)
{
	int loop;
	MD5_CTX ctx;
	unsigned char md5_bin[16];
	char md5_str[32 + 1];

	rss_add_section(into, "item", 1);

	rss_add_string(into, "title", title);
	rss_add_string(into, "link", link);
	rss_add_string(into, "description", description);
	
	(void)MD5_Init(&ctx);
	(void)MD5_Update(&ctx, title, strlen(title));
	(void)MD5_Update(&ctx, link, strlen(link));
	(void)MD5_Update(&ctx, description, strlen(description));
	(void)MD5_Final(md5_bin, &ctx);
	for(loop=0; loop<16; loop++)
	{
		md5_str[loop * 2 + 0] = b2h(md5_bin[loop] >> 4);
		md5_str[loop * 2 + 1] = b2h(md5_bin[loop] & 15);
	}
	md5_str[32] = 0x00;
	rss_add_guid(into, md5_str, 0);

	rss_add_timestamp(into, timestamp);

	rss_add_section(into, "item", 0);
}

void rss_tail(char **into)
{
	int len = strlen(*into);
	const char *what = "</channel></rss>";
	int what_len = strlen(what);

	*into = (char *)myrealloc(*into, len + what_len + 1, "rss_tail");

	sprintf(&(*into)[len], "%s", what);
}
